/*
 * adc.c
 *
 * Created: 19/07/2023 15:33:59
 *  Author: Simon 
 */ 

#include "system.h"

#include "samc21.h"
#include "adc_hal.h"
#include "nvic_hal.h"
#include "clock.h"

#include "port_control.h"


Adc * instance[2] = {ADC0 , ADC1 } ;
	


volatile uint16_t adc_result[2][12] ;       // This array will hold the channel results

#define REG_ADC0_CAL (*(volatile uint32_t*) 0x806020)
#define REG_ADC1_CAL (*(volatile uint32_t*) 0x806021)

volatile uint16_t samplenun_reg, adjres_reg ;

void adc_setup_enable_rr(Adc * instance, ADC_SF_t prescaler, ADC_CH_t channels , adc_resolution_t resolution)
{
	if (instance == ADC0)
	{
		connect_peripheral_to_clock(ADC0_GCLK , F_32MHz) ;
		
		if (channels & ADC_CH0)  pin_mode(PA02 , Analogue) ;
		if (channels & ADC_CH1)  pin_mode(PA03 , Analogue) ;
		if (channels & ADC_CH2)  pin_mode(PB08 , Analogue) ;
		if (channels & ADC_CH3)  pin_mode(PB09 , Analogue) ;
		if (channels & ADC_CH4)  pin_mode(PA04 , Analogue) ;
		if (channels & ADC_CH5)  pin_mode(PA05 , Analogue) ;
		if (channels & ADC_CH6)  pin_mode(PA06 , Analogue) ;
		if (channels & ADC_CH7)  pin_mode(PA07 , Analogue) ;
		if (channels & ADC_CH8)  pin_mode(PA08 , Analogue) ;
		if (channels & ADC_CH9)  pin_mode(PA09 , Analogue) ;
		if (channels & ADC_CH10) pin_mode(PA10 , Analogue) ;
		if (channels & ADC_CH11) pin_mode(PA11 , Analogue) ;
				
		interrupt_enable(25U) ;
	}
	
	if (instance == ADC1)
	{
		connect_peripheral_to_clock(ADC1_GCLK , F_32MHz) ;
		
		if (channels & ADC_CH0)  pin_mode(PB00 , Analogue) ;
		if (channels & ADC_CH1)  pin_mode(PB01 , Analogue) ;
		if (channels & ADC_CH2)  pin_mode(PB02 , Analogue) ;
		if (channels & ADC_CH3)  pin_mode(PB03 , Analogue) ;
		if (channels & ADC_CH4)  pin_mode(PB08 , Analogue) ;
		if (channels & ADC_CH5)  pin_mode(PB09 , Analogue) ;
		if (channels & ADC_CH6)  pin_mode(PB04 , Analogue) ;
		if (channels & ADC_CH7)  pin_mode(PB05 , Analogue) ;
		if (channels & ADC_CH8)  pin_mode(PB06 , Analogue) ;
		if (channels & ADC_CH9)  pin_mode(PB07 , Analogue) ;
		if (channels & ADC_CH10) pin_mode(PA08 , Analogue) ;
		if (channels & ADC_CH11) pin_mode(PA09 , Analogue) ;
		
		interrupt_enable(26U) ;		
	}

	instance->CTRLB.bit.PRESCALER |= prescaler ;

	while (instance->SYNCBUSY.bit.CTRLC) ;
	instance->CTRLC.bit.RESSEL = 0x1 ;

	while (instance->SYNCBUSY.bit.CTRLC) ;
	instance->CTRLC.bit.FREERUN = 0x1 ;

	instance->REFCTRL.bit.REFSEL = 0x5 ;
	
	// put adjres value (<< 4) and samplenunm (<< 0) into avgctrl register
	if (resolution == adc_resolution_12_fast)          instance->AVGCTRL.reg = 0x0 << 0x4 | 0x00 << 0 ;
	if (resolution == adc_resolution_12_filtered_1024) instance->AVGCTRL.reg = 0x2 << 0x4 | 0x0A << 0 ;
	if (resolution == adc_resolution_16_fast)          instance->AVGCTRL.reg = 0x0 << 0x4 | 0x08 << 0 ;
	if (resolution == adc_resolution_16_filtered_4)     instance->AVGCTRL.reg = 0x0 << 0x4 | 0x0A << 0 ;
	
	if (instance == ADC0)
	instance->CALIB.reg =  (uint16_t) ( ( REG_ADC0_CAL & 0x7 ) << 8 | ( REG_ADC0_CAL & 0b00111000 ) >> 3 );

	if (instance == ADC1)
	instance->CALIB.reg =  (uint16_t) ( ( REG_ADC1_CAL & 0x7 ) << 8 | ( REG_ADC1_CAL & 0b00111000 ) >> 3 );

	instance->SEQCTRL.reg |= channels ; // slots selected by ADC_add_channel

	instance->CTRLA.bit.ENABLE = 0x1 ;

	instance->INTENSET.bit.RESRDY = 1 ;
	
	instance->SWTRIG.bit.START = 1 ;
}


void add_adc_channel(Adc * instance , ADC_CH_t channel )
{
	if (instance == ADC0)
	{
		connect_peripheral_to_clock(ADC0_GCLK, F_32MHz) ;
		
		if (channel == ADC_CH0)  pin_mode(PA02 , Analogue) ;
		if (channel == ADC_CH1)  pin_mode(PA03 , Analogue) ;
		if (channel == ADC_CH2)  pin_mode(PB08 , Analogue) ;
		if (channel == ADC_CH3)  pin_mode(PB09 , Analogue) ;
		if (channel == ADC_CH4)  pin_mode(PA04 , Analogue) ;
		if (channel == ADC_CH5)  pin_mode(PA05 , Analogue) ;
		if (channel == ADC_CH6)  pin_mode(PA06 , Analogue) ;
		if (channel == ADC_CH7)  pin_mode(PA07 , Analogue) ;
		if (channel == ADC_CH8)  pin_mode(PA08 , Analogue) ;
		if (channel == ADC_CH9)  pin_mode(PA09 , Analogue) ;
		if (channel == ADC_CH10) pin_mode(PA10 , Analogue) ;
		if (channel == ADC_CH11) pin_mode(PA11 , Analogue) ;
	}
	
	if (instance == ADC1)
	{
		connect_peripheral_to_clock(ADC1_GCLK, F_32MHz) ;
		
		if (channel == ADC_CH0)  pin_mode(PB00 , Analogue) ;
		if (channel == ADC_CH1)  pin_mode(PB01 , Analogue) ;
		if (channel == ADC_CH2)  pin_mode(PB02 , Analogue) ;
		if (channel == ADC_CH3)  pin_mode(PB03 , Analogue) ;
		if (channel == ADC_CH4)  pin_mode(PB08 , Analogue) ;
		if (channel == ADC_CH5)  pin_mode(PB09 , Analogue) ;
		if (channel == ADC_CH6)  pin_mode(PB04 , Analogue) ;
		if (channel == ADC_CH7)  pin_mode(PB05 , Analogue) ;
		if (channel == ADC_CH8)  pin_mode(PB06 , Analogue) ;
		if (channel == ADC_CH9)  pin_mode(PB07 , Analogue) ;
		if (channel == ADC_CH10) pin_mode(PA08 , Analogue) ;
		if (channel == ADC_CH11) pin_mode(PA09 , Analogue) ;
	}
	
	instance->SEQCTRL.reg |= channel ; // slots selected by ADC_add_channel
}

void ADC_rr_handler(Adc * instance)
{
	if (instance == ADC0) adc_result[0][instance->SEQSTATUS.reg & 0x0F] = instance->RESULT.reg ;
	if (instance == ADC1) adc_result[1][instance->SEQSTATUS.reg & 0x0F] = instance->RESULT.reg ;
}