/*
 * can_hal.c
 *
 * Created: 26/06/2023 16:36:20
 *  Author: Simon 
 */

#include "samc21.h"
#include "clock.h"
#include "nvic_hal.h"

static const uint8_t number_rx_filters[2] = {2 , 0} ;
static const uint8_t number_tx_ids[2] = {2 , 0} ;


	
static const uint32_t ram_start_address[2] = {0x0400 , 0 } ;



static const uint32_t rx_std_filter_start_address[2] = {0x1600 , 0} ;
static const uint32_t tx_buffer_start_address[2] = {0x16FF , 0} ;
static const uint32_t gclk_f = 48000000 ; // frequency the controller is clocked from


Can * canport[2] = {CAN0 , CAN1} ;

void can_init(uint8_t instance, uint32_t baud_rate )
{
	connect_peripheral_to_clock((CAN0_GCLK + instance), F_48MHz) ; // connect clock to CPU clock, bridge connection made in clock setup

	canport[instance]->CCCR.bit.INIT = 1 ; // put the can peripheral into initialization mode
	canport[instance]->CCCR.bit.CCE = 1 ; // enable writing of registers while in initialization mode

	uint32_t baud_rate_div = (gclk_f / (baud_rate * 0x16)) - 0x1 ;

	if (baud_rate_div > 511U)
	{
		while(1) ; // source clock is too fast, a slower clock is needed for the baud rate requested or increase the quanta per bit,
	}

	canport[instance]->NBTP.bit.NBRP = baud_rate_div ; //prescaler to obtain the arbitration field quantization clock

	// receiver setup

	canport[instance]->RXBC.reg = ram_start_address[instance] ; // rx buffer address starts at the beginning of the allocated RAM

	canport[instance]->SIDFC.reg = number_rx_filters[instance] << 16 | rx_std_filter_start_address[instance] ; // number of standard Rx filters and
	// start address in ram leaving 1152 words for the Rx buffers

	// transmitter setup

	canport[instance]->TXBC.bit.NDTB = number_tx_ids[instance] ; // dedicated transmit buffers

	canport[instance]->TXBC.bit.TBSA = tx_buffer_start_address[instance] ;

	// enable the CAN peripheral

	interrupt_enable(NVIC_INT_CAN0_CH + instance) ;

	canport[instance]->CCCR.bit.INIT = 0 ; // conclude can configuration
}

#define filter_slot_address(x) (*(volatile uint32_t*)(rx_std_filter_start_address[instance] + filter_slot * 0x4 ))

void add_can11_id_filter(uint8_t instance , uint8_t filter_slot , uint16_t standard_filter_id , uint8_t rx_buffer_slot)
{
	filter_slot_address(filter_slot) = 0x2 << 30 | 0x7 << 27 | standard_filter_id << 16 |
	 0x00 << 9 | 0x00 << 6 | (rx_buffer_slot * 4) << 0 ;
}

void send_can11_message(uint8_t instance , uint16_t id , void * payload_data_pointer, uint8_t payload_size )
{

}