/*
 * clock.c
 *
 * Created: 20/07/2023 15:01:12
 *  Author: Simon
 */ 

// peripheral clock sources,  16.8.4

#include "samc21.h"
#include "clock.h"

void connect_peripheral_to_clock(peripheral_clock_channel_t peripheral, clock_source_t gclk_channel)
{
	GCLK->PCHCTRL[peripheral].reg = 0x1 << 6 | gclk_channel << 0 ;
}

void clock_setup ()
{
	// External 32.768kHz crystal
	
	// REG_OSC32KCTRL_XOSC32K = 0x2 << 8 | 0x1 << 3 | 0x1 << 2 | 0x1 << 1 ; // replace by next line
	OSC32KCTRL->XOSC32K.reg = 0x5 << 8 | 0x1 << 3 | 0x1 << 2 | 0x1 << 1 ;
	while ( ( REG_OSC32KCTRL_STATUS & 0x1 ) != 0x1 ) ;
	// REG_OSC32KCTRL_CFDCTRL = 0x1 << 1 | 0x1 << 0 ; // replace by next line
	OSC32KCTRL->CFDCTRL.reg = 0x1 << 1 | 0x1 << 0 ;

	OSC32KCTRL->RTCCTRL.bit.RTCSEL = 0x5 ; // RTC clock selection

	// DPLL 20.8.12-17

	REG_OSCCTRL_DPLLRATIO = 11 << 16 | 2928 << 0 ; // multiply 32.768 kHz by 2929.6875 to 96MHz 20.8.13
	while ( OSCCTRL->DPLLSYNCBUSY.bit.ENABLE ) ;

	// control B reg. is all defaults 0x0

	REG_OSCCTRL_DPLLCTRLA = 0x1 << 1 ; // enable DPLL96M 20.8.12
	while ( REG_OSCCTRL_DPLLSTATUS  != 0x3 ) ;

	// NVMCTRL 27.8.2

	NVMCTRL->CTRLB.reg = 0x2 << 1 ; // nvmctrl will have 2 wait states to support 48MHz MCLK

	// CPU CLK

	MCLK->CPUDIV = 0x1 << 0 ; // CPU clock divider is set to 1 for 48MHz

	// GCLK GENERATORS 16.8.3

	// division_factor << 16 | run_in_standby << 13 | divide_by_power_enable << 12 | connect_clock_to_pin << 11 | output_state << 10 | improve_duty_cycle << 9 | enable << 8 | source << 0 ;

	GCLK->GENCTRL[0].reg = 0x0002 << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x7 << 0 ; // Gen0 48MHz from DPLL96M
	//GCLK->GENCTRL[0].reg = 0x0001 << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x5 << 0 ; // Gen0 32kHz from 32kHz
	GCLK->GENCTRL[1].reg = 0x3 << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x7 << 0 ; // Gen1 32MHz for ADC from DPLL96M
	GCLK->GENCTRL[2].reg = 0x1 << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x7 << 0 ; // Gen2 96MHz clk for TCC0/1 and frequency measurement from DPLL96M
	GCLK->GENCTRL[3].reg = 0x1 << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x5 << 0 ; // Gen3 32.768 kHz crystal
	GCLK->GENCTRL[4].reg = 20U << 16 | 0x1 << 13 | 0x0 << 12 | 0x0 << 11 | 0x0 << 10 | 0x1 << 8 | 0x2 << 0 ; // Gen4 2.4 MHz


	// enable all peripherals clocks others are already enabled with only some exceptions

	MCLK->APBCMASK.reg = 0xFFFFFFFF ;
	MCLK->AHBMASK.reg = 0xFFFFFFFF ;
}
