/*
 * eic.h
 *
 * Created: 16/06/2023 10:42:05
 *  Author: Simon
 */ 


#ifndef EIC_H_
#define EIC_H_

#include "samc21.h"
#include "clock.h"
#include "port_control.h"
#include "nvic_hal.h"

 typedef enum {
	none = 0x0 ,
	risingEdge = 0x1 ,
	fallingEdge = 0x2 ,
	eitherEdge = 0x3 ,
	highLevel = 0x4 ,
	lowLevel = 0x5
} eic_trigger_signal_t;

typedef enum {
	interruptEnabled = 0x1 ,
	noInterrupt = 0x0
} eic_interrupt_enable_t;

typedef enum {
	eventEnable = 1 ,
	noEvent = 0
} eic_event_enable_t ;

static inline void eic_enable()
{
	interrupt_enable(NVIC_INT_EIC_CH) ;

	EIC->CTRLA.bit.ENABLE = 0x1 ;
}

static inline void eic_pin_setup(uint8_t portPin, eic_trigger_signal_t action_on, eic_interrupt_enable_t interrupt_enable, eic_event_enable_t event_enable)
{
	uint8_t portn = portPin >> 5 ; // dump pin number section leaving port number
	uint8_t pin_number = portPin % 32 ; // dump port number leaving pin number
	static uint16_t used_channels ;

	if (used_channels & (0x1 << (portPin % 16)))
	{
		while (1)
		{
			;// you have already used a channel
		}
	}
	else used_channels |= 0x1 << (portPin % 16) ;

	if (portPin % 2) // if the pin is odd numbered
	{
		PORT->Group[portn].PMUX[pin_number/2].bit.PMUXO = 0x0 ; //  fill in multiplexer
	}
	else // else the pin is even numbered
	{
		PORT->Group[portn].PMUX[pin_number/2].bit.PMUXE = 0x0 ; //  fill in multiplexer
	}

	PORT->Group[portn].PINCFG[pin_number].bit.PMUXEN = 0x1 ; // enable pin multiplexer

	if (interrupt_enable) EIC->INTENSET.reg = 0x1 << (portPin % 16) ; // if applicable enable EIC interrupt

	if (event_enable) EIC->EVCTRL.reg |= 0x1 << (portPin % 16) ; //  if applicable enable EIC event

	EIC->CONFIG[ (portPin & 0xF) / 0x8 ].reg |= action_on << ((portPin % 8) *4) ; // configure events
}


#endif /* EIC_H_ */