/*
 * nvic_hal.h
 *
 * Created: 27/06/2023 12:00:40
 *  Author: Simon
 */ 


#ifndef NVIC_HAL_H_
#define NVIC_HAL_H_

typedef enum{
 NVIC_INT_SYS_CH     = 0 ,
 NVIC_INT_WDT_CH     = 1 ,
 NVIC_INT_RTC_CH     = 2 ,
 NVIC_INT_EIC_CH     = 3 ,

 NVIC_INT_FREQM_CH   = 4 ,
 NVIC_INT_TSENSE_CH  = 5 ,
 NVIC_INT_NVMCTRL_CH = 6 ,
 NVIC_INT_DMAC_CH    = 7 ,

 NVIC_INT_EVSYS_CH   = 8 ,
 NVIC_INT_SERCOM0_CH = 9 ,
 NVIC_INT_SERCOM6_CH = 9 ,
 NVIC_INT_SERCOM1_CH = 10 ,
 NVIC_INT_SERCOM7_CH = 10 ,
 NVIC_INT_SERCOM2_CH = 11 ,

 NVIC_INT_SERCOM3_CH = 12 ,
 NVIC_INT_SERCOM4_CH = 13 ,
 NVIC_INT_SERCOM5_CH = 14 ,
 NVIC_INT_CAN0_CH    = 15 ,

 NVIC_INT_CAN1_CH    = 16 ,
 NVIC_INT_TCC0_CH    = 17 ,
 NVIC_INT_TCC1_CH    = 18 ,
 NVIC_INT_TCC2_CH    = 19 ,

 NVIC_INT_TC0_CH     = 20 ,
 NVIC_INT_TC5_CH     = 20 ,
 NVIC_INT_TC1_CH     = 21 ,
 NVIC_INT_TC6_CH     = 21 ,
 NVIC_INT_TC2_CH     = 22 ,
 NVIC_INT_TC7_CH     = 22 ,
 NVIC_INT_TC3_CH     = 23 ,

 NVIC_INT_TC4_CH     = 24 ,
 NVIC_INT_ADC0_CH    = 25 ,
 NVIC_INT_ADC1_CH    = 26 ,
 NVIC_INT_AC_CH      = 27 ,

 NVIC_INT_DAC_CH     = 28 ,
 NVIC_INT_SDADC_CH   = 29 ,
 NVIC_INT_PTC_CH     = 30
 
	} nvic_channel_number_t;

#define REG_NVIC_INT_EN (*(volatile uint32_t*)(0xE000E100))

static inline void interrupt_enable(nvic_channel_number_t n) /* NVIC_INT_enable(n);  load a 1 into any of the n positions to enable 0's have no effect */
{
	REG_NVIC_INT_EN = 0x1 << n;
}

static inline void intterupt_disable(nvic_channel_number_t n) /* load a 1 into any of the above positions to disable NVIC_INT_disable(n); */
{
	(*(volatile uint32_t*)(0xE000E180)) = 0x1 << n;
}

static inline void interrupt_manual_trigger(nvic_channel_number_t n) /* load a 1 into any of the above positions to manually trigger an interrupt on the NVIC_INT_manual(n); */
{
	(*(volatile uint32_t*)(0xE000E200)) = 0x1 << n;
}

static inline void interrupt_rearm(nvic_channel_number_t n) /* load a 1 into any of the above positions to rearm an iterrupt on the NVIC_INT_rearm(n);
	(this will not be required for normal interrupt servicing) */
{
	(*(volatile uint32_t*)(0xE000E280)) = 0x1 << n;
}

static inline void interrupt_channel_priority(uint8_t ch , uint8_t priority) /* set the priority of the named channel NVIC_INT_PER_priority(ch, priority); */
{
	(*(volatile uint32_t*)(0xE000E400 + (ch / 4) ) ) |= (priority << 6) << ((ch % 4) * 8);
	/* there are 4 channels per register with 8 bits used for each channel, bits 6 & 7 of each 8 bit set are the 2 priority bits that set a priority of 0-3
	with 0 having the highest priority */
}

static inline void interrupt_batch_priority(uint8_t ch, uint32_t n) /* set the priority of the 4 channels in the same register as the channel called NVIC_INT_priority(ch, priority); */
{
	(*(volatile uint32_t*)(0xE000E400 + ch / 4)) = n;
}




#endif /* NVIC_HAL_H_ */