/*
 * port_control.c
 *
 * Created: 19/07/2023 15:40:28
 *  Author: Simon
 */ 

#include "system.h"
#include "samc21.h"

#include "port_control.h"
#include "port_defs.h"

uint32_t port_pin_in_use[2] ;

void pin_mode(pin_t pin_number, port_pin_functions_t mux_selection)
{
	#ifdef DEBUG_ON
	allocate_port_pin(pin_number) ; // check to see if pin has already been allocated
	#endif
	
	if (mux_selection > 0x0F) // check for number higher than any of the multiplexed mode (GPIO)
	{
		if (mux_selection == Digital_output)
		{
			PORT->Group[pin_number >> 5].DIRSET.reg = 0x1 << (pin_number & 31U) ;
		}
		
		if (mux_selection == Digital_input)
		{
			PORT->Group[pin_number >> 5].DIRCLR.reg = 0x1 << (pin_number & 31U) ;
		}
	}
	else // otherwise setup the pin multiplexer
	{
		uint8_t portn = pin_number >> 5 ; // bit 5 and up is the port number
		uint8_t pin = pin_number & 31U ; // take out the port number from bit 5 onwards
		
		if (pin % 2)
		{
			PORT->Group[portn].PMUX[pin/2].bit.PMUXO = mux_selection;
		}
		else
		{
			PORT->Group[portn].PMUX[pin/2].bit.PMUXE = mux_selection;
		}
		
		PORT->Group[portn].PINCFG[pin].bit.PMUXEN = 0x1 ;
	}
}

void pin_set(uint8_t pin)
{
	PORT->Group[pin >> 5].OUTSET.reg = 0x1 << (pin & 31U) ;
}

void pin_clear(uint8_t pin)
{
	PORT->Group[pin >> 5].OUTCLR.reg = 0x1 << (pin & 31U) ;
}

void pin_toggle(uint8_t pin)
{
	PORT->Group[pin >> 5].OUTTGL.reg = 0x1 << (pin & 31U) ;
}

void pin_pullup(uint8_t pin)
{
	PORT->Group[pin >> 5].PINCFG->bit.PULLEN = 0x1 << (pin & 31U) ;
	pin_set( pin ) ;
}

void pin_pulldown(uint8_t pin)
{
	PORT->Group[pin >> 5].PINCFG->bit.PULLEN = 0x1 << (pin & 31U) ;
	pin_clear( pin ) ;
}

uint32_t pinRead(uint8_t pin)
{
	return PORT->Group[pin >> 5].IN.reg & (0x1 << (pin & 31U)) ;
}


void allocate_port_pin(uint8_t port_pin)
{
	uint8_t port = port_pin >> 5 ;
	uint8_t  pin = port_pin & 0x31 ;	

	if ( port_pin_in_use[port] & (0x1 << pin) )
	{
		while (1)
		{
			; // this is the second time you are using this pin
		} 
	}
	
	else port_pin_in_use[port] |= (0x1 << pin) ; 
}


/*
void pinFunction(uint8_t pin_number,  uint8_t function)
{
	allocate_port_pin(pin_number) ;
	
	uint8_t portn = pin_number >> 5 ; // bit 5 and up is the port number
	uint8_t pin = pin_number & 31U ; // take out the port number from bit 5 onwards
	
	if (pin %2)
	{
		PORT->Group[portn].PMUX[pin/2].bit.PMUXO = function ;
	}
	else
	PORT->Group[portn].PMUX[pin/2].bit.PMUXE = function ;

	PORT->Group[portn].PINCFG[pin].bit.PMUXEN = 0x1 ;
}*/